@@ -21,18 +21,18 @@ class LZ4FrameCompressor(object):
         block_size (int): Sepcifies the maximum blocksize to use.
             Options:
 
-                - lz4.frame.BLOCKSIZE_DEFAULT or 0: the lz4 library default
-                - lz4.frame.BLOCKSIZE_MAX64KB or 4: 64 kB
-                - lz4.frame.BLOCKSIZE_MAX256KB or 5: 256 kB
-                - lz4.frame.BLOCKSIZE_MAX1MB or 6: 1 MB
-                - lz4.frame.BLOCKSIZE_MAX4MB or 7: 4 MB
+                - `lz4.frame.BLOCKSIZE_DEFAULT` or 0: the lz4 library default
+                - `lz4.frame.BLOCKSIZE_MAX64KB` or 4: 64 kB
+                - `lz4.frame.BLOCKSIZE_MAX256KB` or 5: 256 kB
+                - `lz4.frame.BLOCKSIZE_MAX1MB` or 6: 1 MB
+                - `lz4.frame.BLOCKSIZE_MAX4MB` or 7: 4 MB
 
-            If unspecified, will default to lz4.frame.BLOCKSIZE_DEFAULT which
-            is equal to lz4.frame.BLOCKSIZE_MAX64KB.
+            If unspecified, will default to `lz4.frame.BLOCKSIZE_DEFAULT` which
+            is equal to `lz4.frame.BLOCKSIZE_MAX64KB`.
         block_linked (bool): Specifies whether to use block-linked
-            compression. If `True`, the compression ratio is improved, especially
-            for small block sizes. If `False` the blocks are compressed independently.
-            The default is `True`.
+            compression. If ``True``, the compression ratio is improved, especially
+            for small block sizes. If ``False`` the blocks are compressed independently.
+            The default is ``True``.
         compression_level (int): Specifies the level of compression used.
             Values between 0-16 are valid, with 0 (default) being the
             lowest compression (0-2 are the same value), and 16 the highest.
@@ -40,29 +40,29 @@ class LZ4FrameCompressor(object):
             Values between 4-9 are recommended. 0 is the default.
             The following module constants are provided as a convenience:
 
-                - lz4.frame.COMPRESSIONLEVEL_MIN: Minimum compression (0)
-                - lz4.frame.COMPRESSIONLEVEL_MINHC: Minimum high-compression (3)
-                - lz4.frame.COMPRESSIONLEVEL_MAX: Maximum compression (16)
+                - `lz4.frame.COMPRESSIONLEVEL_MIN`: Minimum compression (0)
+                - `lz4.frame.COMPRESSIONLEVEL_MINHC`: Minimum high-compression (3)
+                - `lz4.frame.COMPRESSIONLEVEL_MAX`: Maximum compression (16)
 
         content_checksum (bool): Specifies whether to enable checksumming of
-            the payload content. If `True` a checksum of the uncompressed data
+            the payload content. If ``True`` a checksum of the uncompressed data
             is stored at the end of the compressed frame which is checked during
-            decompression. The default is `False`.
+            decompression. The default is ``False``.
         block_checksum (bool): Specifies whether to enable checksumming of
-            the content of each block. If `True` a checksum of the uncompressed
+            the content of each block. If ``True`` a checksum of the uncompressed
             data in each block in the frame is stored at the end of each block.
             If present, these checksums will be used to validate the data during
-            decompression. The default is `False`, meaning block checksums are not
+            decompression. The default is ``False``, meaning block checksums are not
             calculated and stored. This functionality is only supported if the
             underlying LZ4 library has version >= 1.8.0. Attempting to set this
-            value to `True` with a version of LZ4 < 1.8.0 will cause a RuntimeError
+            value to `True` with a version of LZ4 < 1.8.0 will cause a ``RuntimeError``
             to be raised.
-        auto_flush (bool): When `False`, the LZ4 library may buffer data until a
-            block is full. When `True` no buffering occurs, and partially full
-            blocks may be returned. The default is `False`.
-        return_bytearray (bool): When `False` a bytes object is returned from the
-            calls to methods of this class. When `True` a bytearray object will be
-            returned. The default is `False`.
+        auto_flush (bool): When ``False``, the LZ4 library may buffer data until a
+            block is full. When ``True`` no buffering occurs, and partially full
+            blocks may be returned. The default is ``False``.
+        return_bytearray (bool): When ``False`` a ``bytes`` object is returned from the
+            calls to methods of this class. When ``True`` a ``bytearray`` object will be
+            returned. The default is ``False``.
     """
 
     def __init__(self,
@@ -84,7 +84,7 @@ class LZ4FrameCompressor(object):
         self.block_checksum = block_checksum
         self.auto_flush = auto_flush
         self.return_bytearray = return_bytearray
-        self._context = create_compression_context()
+        self._context = None
         self._started = False
 
     def __enter__(self):
@@ -101,7 +101,7 @@ class LZ4FrameCompressor(object):
         information. The data returned from subsequent calls to ``compress()``
         should be concatenated with this header.
 
-        Args:
+        Keyword Args:
             source_size (int): Optionally specify the total size of the
                 uncompressed data. If specified, will be stored in the
                 compressed frame header as an 8-byte field for later use
@@ -112,6 +112,7 @@ class LZ4FrameCompressor(object):
         """
 
         if self._started is False:
+            self._context = create_compression_context()
             result = compress_begin(
                 self._context,
                 block_size=self.block_size,
@@ -126,7 +127,9 @@ class LZ4FrameCompressor(object):
             self._started = True
             return result
         else:
-            raise RuntimeError('compress_begin called when not already initialized')
+            raise RuntimeError(
+                'LZ4FrameCompressor.begin() called after already initialized'
+            )
 
     def compress(self, data):
         """Compress ``data`` (a ``bytes`` object), returning a bytes object
@@ -179,11 +182,20 @@ class LZ4FrameCompressor(object):
         self._started = False
         return result
 
+    def flush(self):
+        """This function is identical to `LZ4Compressor.finalize()` and is provided for
+        API compatibility with the compressors included in the Python standard
+        library.
+
+        """
+        result = finalize()
+        return result
+
     def reset(self):
         """Reset the LZ4FrameCompressor instance allowing it to be re-used
 
         """
-        self._context = create_compression_context()
+        self._context = None
         self._started = False
 
 
@@ -199,7 +211,6 @@ class LZ4FrameDecompressor(object):
             calls to methods of this class. When ``True`` a bytearray object will be
             returned. The default is ``False``.
 
-
     Attributes:
         eof (bool): ``True`` if the end-of-stream marker has been reached. ``False``
             otherwise.
@@ -215,7 +226,7 @@ class LZ4FrameDecompressor(object):
         self._context = create_decompression_context()
         self.eof = False
         self.needs_input = True
-        self.unused_data = b''
+        self.unused_data = None
         self._unconsumed_data = b''
         self._return_bytearray = return_bytearray
 
@@ -228,23 +239,41 @@ class LZ4FrameDecompressor(object):
         # so no need to del it here
         pass
 
+    def reset(self):
+        """Reset the decompressor state. This is useful after an error occurs, allowing
+        re-use of the instance.
+
+        """
+        reset_decompression_context(self._context)
+        self.eof = False
+        self.needs_input = True
+        self.unused_data = None
+        self._unconsumed_data = b''
+
+
     def decompress(self, data, max_length=-1):
         """Decompresses part or all of an LZ4 frame of compressed data. The returned
         data should be concatenated with the output of any previous calls to
-        ``decompress()``.
+        `decompress()`.
 
         If ``max_length`` is non-negative, returns at most ``max_length`` bytes
         of decompressed data. If this limit is reached and further output can
-        be produced, the ``needs_input`` attribute will be set to ``False``. In
-        this case, the next call to ``decompress()`` may provide data as
+        be produced, the `needs_input` attribute will be set to ``False``. In
+        this case, the next call to `decompress()` may provide data as
         ``b''`` to obtain more of the output. In all cases, any unconsumed data
         from previous calls will be prepended to the input data.
 
         If all of the input ``data`` was decompressed and returned (either
         because this was less than ``max_length`` bytes, or because
-        ``max_length`` was negative), the ``needs_input`` attribute will be set
+        ``max_length`` was negative), the `needs_input` attribute will be set
         to ``True``.
 
+        If an end of frame marker is encountered in the data during
+        decompression, decompression will stop at the end of the frame, and any
+        data after the end of frame is available from the `unused_data`
+        attribute. In this case, the `LZ4FrameDecompressor` instance is reset
+        and can be used for further decompression.
+
         Args:
             data (str, bytes or buffer-compatible object): compressed data to
                 decompress
@@ -277,6 +306,7 @@ class LZ4FrameDecompressor(object):
         else:
             self._unconsumed_data = b''
             self.needs_input = True
+            self.unused_data = None
 
         self.eof = eoframe
 
@@ -299,7 +329,7 @@ class LZ4FrameFile(_compression.BaseStream):
     returned as bytes, and data to be written must be given as bytes.
 
     When opening a file for writing, the settings used by the compressor can be
-    specified. The underlying compressor object is ``lz4.frame.LZ4FrameCompressor``.
+    specified. The underlying compressor object is `lz4.frame.LZ4FrameCompressor`.
     See the docstrings for that class for details on compression options.
 
     Args:
@@ -309,31 +339,32 @@ class LZ4FrameFile(_compression.BaseStream):
             can be an existing file object to read from or write to.
 
     Keyword Args:
-        mode(str): mode can be "r" for reading (default), "w" for (over)writing,
-            "x" for creating exclusively, or "a" for appending. These can
-            equivalently be given as "rb", "wb", "xb" and "ab" respectively.
-        return_bytearray (bool): When ``False`` a bytes object is returned from the
-            calls to methods of this class. When ``True`` a bytearray object will be
-            returned. The default is ``False``.
+        mode(str): mode can be ``'r'`` for reading (default), ``'w'`` for
+            (over)writing, ``'x'`` for creating exclusively, or ``'a'``
+            for appending. These can equivalently be given as ``'rb'``, ``'wb'``,
+            ``'xb'`` and ``'ab'`` respectively.
+        return_bytearray (bool): When ``False`` a bytes object is returned from
+            the calls to methods of this class. When ``True`` a ``bytearray``
+            object will be returned. The default is ``False``.
         source_size (int): Optionally specify the total size of the uncompressed
             data. If specified, will be stored in the compressed frame header as
-            an 8-byte field for later use during decompression. Default is 0
+            an 8-byte field for later use during decompression. Default is ``0``
             (no size stored). Only used for writing compressed files.
-        block_size (int): Compressor setting. See ``lz4.frame.LZ4FrameCompressor``.
+        block_size (int): Compressor setting. See `lz4.frame.LZ4FrameCompressor`.
         block_linked (bool): Compressor setting. See
-            ``lz4.frame.LZ4FrameCompressor``.
+            `lz4.frame.LZ4FrameCompressor`.
         compression_level (int): Compressor setting. See
-            ``lz4.frame.LZ4FrameCompressor``.
+            `lz4.frame.LZ4FrameCompressor`.
         content_checksum (bool): Compressor setting. See
-            ``lz4.frame.LZ4FrameCompressor``.
+            `lz4.frame.LZ4FrameCompressor`.
         block_checksum (bool): Compressor setting. See
-            ``lz4.frame.LZ4FrameCompressor``.
+            `lz4.frame.LZ4FrameCompressor`.
         auto_flush (bool): Compressor setting. See
-            ``lz4.frame.LZ4FrameCompressor``.
+            `lz4.frame.LZ4FrameCompressor`.
 
     """
 
-    def __init__(self, filename=None, mode="r",
+    def __init__(self, filename=None, mode='r',
                  block_size=BLOCKSIZE_DEFAULT,
                  block_linked=True,
                  compression_level=COMPRESSIONLEVEL_MIN,
@@ -347,9 +378,9 @@ class LZ4FrameFile(_compression.BaseStream):
         self._closefp = False
         self._mode = _MODE_CLOSED
 
-        if mode in ("r", "rb"):
+        if mode in ('r', 'rb'):
             mode_code = _MODE_READ
-        elif mode in ("w", "wb", "a", "ab", "x", "xb"):
+        elif mode in ('w', 'wb', 'a', 'ab', 'x', 'xb'):
             mode_code = _MODE_WRITE
             self._compressor = LZ4FrameCompressor(
                 block_size=block_size,
@@ -362,7 +393,7 @@ class LZ4FrameFile(_compression.BaseStream):
             )
             self._pos = 0
         else:
-            raise ValueError("Invalid mode: {!r}".format(mode))
+            raise ValueError('Invalid mode: {!r}'.format(mode))
 
         if sys.version_info > (3, 6):
             path_test = isinstance(filename, (str, bytes, os.PathLike))
@@ -370,16 +401,16 @@ class LZ4FrameFile(_compression.BaseStream):
             path_test = isinstance(filename, (str, bytes))
 
         if path_test is True:
-            if "b" not in mode:
-                mode += "b"
+            if 'b' not in mode:
+                mode += 'b'
             self._fp = builtins.open(filename, mode)
             self._closefp = True
             self._mode = mode_code
-        elif hasattr(filename, "read") or hasattr(filename, "write"):
+        elif hasattr(filename, 'read') or hasattr(filename, 'write'):
             self._fp = filename
             self._mode = mode_code
         else:
-            raise TypeError("filename must be a str, bytes, file or PathLike object")
+            raise TypeError('filename must be a str, bytes, file or PathLike object')
 
         if self._mode == _MODE_READ:
             raw = _compression.DecompressReader(self._fp, LZ4FrameDecompressor)
@@ -463,8 +494,8 @@ class LZ4FrameFile(_compression.BaseStream):
     def read(self, size=-1):
         """Read up to ``size`` uncompressed bytes from the file.
 
-        If ``size`` is negative or omitted, read until EOF is reached. Returns
-        ``b''`` if the file is already at EOF.
+        If ``size`` is negative or omitted, read until ``EOF`` is reached.
+        Returns ``b''`` if the file is already at ``EOF``.
 
         Args:
             size(int): If non-negative, specifies the maximum number of
@@ -628,15 +659,15 @@ def open(filename, mode="rb",
             (no size stored). Only used for writing compressed files.
         block_size (int): Compressor setting. See ``lz4.frame.LZ4FrameCompressor``.
         block_linked (bool): Compressor setting. See
-            ``lz4.frame.LZ4FrameCompressor``.
+            `lz4.frame.LZ4FrameCompressor`.
         compression_level (int): Compressor setting. See
-            ``lz4.frame.LZ4FrameCompressor``.
+            `lz4.frame.LZ4FrameCompressor`.
         content_checksum (bool): Compressor setting. See
-            ``lz4.frame.LZ4FrameCompressor``.
+            `lz4.frame.LZ4FrameCompressor`.
         block_checksum (bool): Compressor setting. See
-            ``lz4.frame.LZ4FrameCompressor``.
+            `lz4.frame.LZ4FrameCompressor`.
         auto_flush (bool): Compressor setting. See
-            ``lz4.frame.LZ4FrameCompressor``.
+            `lz4.frame.LZ4FrameCompressor`.
     """
     if "t" in mode:
         if "b" in mode:
@@ -661,7 +692,7 @@ def open(filename, mode="rb",
         return_bytearray=return_bytearray,
     )
 
-    if "t" in mode:
+    if 't' in mode:
         return io.TextIOWrapper(binary_file, encoding, errors, newline)
     else:
         return binary_file
